<?php

declare(strict_types=1);

namespace Drupal\billwerk_subscriptions_manage\Controller;

use Drupal\billwerk_subscriptions\EmbedHelper;
use Drupal\billwerk_subscriptions\Subscriber;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for Billwerk User routes.
 */
final class UserSubscriptionSelfserviceController extends ControllerBase {

	/**
	 * {@inheritdoc}
	 */
	public static function create(ContainerInterface $container) {
		return new static(
			$container->get('billwerk_subscriptions.embed_helper'),
		);
	}

	public function __construct(
		protected readonly EmbedHelper $embedHelper) {
	}

	/**
	 * Shows the self service portal for a single subscription of the given user.
	 *
	 * @param \Drupal\user\UserInterface $user
	 *   The user.
	 * @param string $subscriptionId
	 *   The subscription id.
	 *
	 * @return array
	 *   The Drupal render array.
	 */
	public function manageUserContract(UserInterface $user): array {
		$subscriber = Subscriber::load($user);
		$build = $this->embedHelper->buildSelfserviceManageContractEmbed($subscriber);

    $build['user_subscription_selfservice'] = [
      '#type' => 'link',
      '#title' => $this->t('Change my subscription plan & add-ons'),
      '#url' => Url::fromRoute('billwerk_subscriptions_manage.current_user_subscription'),
      '#attributes' => [
        'class' => [
          'button',
          'button--user-subscription-selfservice',
        ]
      ],
    ];

		return $build;
	}

	/**
	 * Checks access for a specific request.
	 *
	 * @param \Drupal\Core\Session\AccountInterface $account
	 *   Run access checks for this account.
	 *
	 * @return \Drupal\Core\Access\AccessResultInterface
	 *   The access result.
	 */
	public function accessManageUserContract(AccountInterface $account, UserInterface $user) {
    $access = AccessResult::forbidden();
		if ($account->id() === $user->id()) {
			// Is the current users account.
			$access = AccessResult::allowedIfHasPermission($account, 'billwerk_subscriptions_selfservice_manage_own_contract');
		}
		else {
			// Is a different users account.
			$access = AccessResult::allowedIfHasPermission($account, 'billwerk_subscriptions_selfservice_manage_any_contract');
		}
    return $access->andIf(AccessResult::allowedIf(Subscriber::loadByDrupalUid((int) $user->id())->hasUserBillwerkContractId()));
	}

}
